<?php

namespace FiloBlu\CategoryPagination\Block\Adminhtml\System\Config;

/**
 * Class PaginationMapping
 * @package FiloBlu\CategoryPagination\Block\Adminhtml\System\Config
 */
class PaginationMapping extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{

    /**
     * Grid columns
     *
     * @var array
     */
    protected $_columns = [];
    /**
     * @var
     */
    protected $_modeRenderer;
    /**
     * @var
     */
    protected $_pageRenderer;
    /**
     * Enable the "Add after" button or not
     *
     * @var bool
     */
    protected $_addAfter = true;
    /**
     * Label of add button
     *
     * @var string
     */
    protected $_addButtonLabel;

    /**
     * Render array cell for prototypeJS template
     *
     * @param string $columnName
     * @return string
     * @throws \Exception
     */
    public function renderCellTemplate($columnName)
    {
        if ($columnName == "mode") {
            //            $this->_columns[$columnName]['class'] = 'input-text required-entry validate-number';
            $this->_columns[$columnName]['style'] = 'width:100px;';
        }
        return parent::renderCellTemplate($columnName);
    }

    /**
     * Check if columns are defined, set template
     *
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Prepare to render
     *
     * @return void
     */
    protected function _prepareToRender()
    {
        $this->addColumn('mode', array(
            'label' => __('Mode'),
            'renderer' => $this->getModeRenderer('mode'),
        ));

        $this->addColumn('page', array(
            'label' => __('Page'),
            'renderer' => $this->getPageRenderer('page'),
        ));

        $this->addColumn('productnumber', array(
            'label' => __('Product N.'),
            'style' => 'max-width: 50px;',
        ));

        $this->addColumn('categoryids', array(
            'label' => __('Category Ids'),
            'style' => 'max-width: 400px;',
        ));

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Returns renderer for country element
     *
     * @return \Magento\Braintree\Block\Adminhtml\Form\Field\Countries
     */
    protected function getModeRenderer()
    {
        if (!$this->_modeRenderer) {
            $this->_modeRenderer = $this->getLayout()->createBlock(
                '\FiloBlu\CategoryPagination\Block\Adminhtml\System\Config\Form\Field\SelectMode', '', ['data' => ['is_render_to_js_template' => true]]
            );
        }
        return $this->_modeRenderer;
    }

    /**
     * @return mixed
     */
    protected function getPageRenderer()
    {
        if (!$this->_pageRenderer) {
            $this->_pageRenderer = $this->getLayout()->createBlock(
                '\FiloBlu\CategoryPagination\Block\Adminhtml\System\Config\Form\Field\SelectPage', '', ['data' => ['is_render_to_js_template' => true]]
            );
        }
        return $this->_pageRenderer;
    }

    /**
     * @param \Magento\Framework\DataObject $row
     */
    protected function _prepareArrayRow(\Magento\Framework\DataObject $row)
    {
        $mode = $row->getMode();
        $page = $row->getPage();
        $options = [];

        $options['option_' . $this->getModeRenderer()->calcOptionHash($mode)] = 'selected="selected"';
        $options['option_' . $this->getPageRenderer()->calcOptionHash($page)] = 'selected="selected"';

        $row->setData('option_extra_attrs', $options);
    }
}
