<?php

namespace FiloBlu\CategoryPagination\Helper;

/**
 * Class Data
 * @package FiloBlu\CategoryPagination\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;
    /**
     * @var \Magento\Catalog\Model\Layer\Resolver
     */
    protected $layerResolver;

    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\Layer\Resolver $layerResolver
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\Layer\Resolver $layerResolver
    )
    {
        $this->request = $request;
        $this->registry = $registry;
        $this->layerResolver = $layerResolver;

        parent::__construct($context);
    }

    /**
     * @return mixed
     */
    public function getPageSizeForCategoryGrid()
    {
        $categoryId = -1;

        // to avoid problems on search
        if ($this->registry->registry('current_category')) {
            $categoryId = $this->registry->registry('current_category')->getId();
        }

        return $this->getProductNumber('grid', (int)$this->request->getParam('p', 1), $categoryId);
    }

    /**
     * TODO: Optimize code
     * @param $mode
     * @param $page
     * @param $currentCategoryId
     * @return mixed
     */
    public function getProductNumber($mode, $page, $currentCategoryId)
    {

        // This if for get default config if filters are applied
        if ($this->getConfig('general/default_config_onfilter') && $this->isFilterApplied()) {
            switch ($mode) {
                case 'grid':
                    return $this->getConfig('frontend/grid_per_page', 'catalog');
                    break;
                case 'list':
                    return $this->getConfig('frontend/list_per_page', 'catalog');
                    break;
                default:
                    break;
            }
        }

        $paginationMapping = unserialize($this->getConfig('general/pagination_mapping'));

        switch ($mode) {
            case 'grid':
                if ($paginationMapping && count($paginationMapping)) {

                    if ($page == 1) {

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'grid') && ($map['page'] == '1') && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == '1') && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'grid') && ($map['page'] == '1') && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == '1') && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        return $this->getConfig('frontend/grid_per_page', 'catalog');
                    } else {

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'grid') && ($map['page'] == "1+") && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == "1+") && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'grid') && ($map['page'] == "1+") && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == "1+") && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        return $this->getConfig('frontend/grid_per_page', 'catalog');
                    }
                } else {
                    return $this->getConfig('frontend/grid_per_page', 'catalog');
                }

                break;
            case 'list':
                if ($paginationMapping && count($paginationMapping)) {
                    if ($page == 1) {

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'list') && ($map['page'] == '1') && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == '1') && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'list') && ($map['page'] == '1') && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == '1') && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        return $this->getConfig('frontend/list_per_page', 'catalog');
                    } else {

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'list') && ($map['page'] == "1+") && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == "1+") && (in_array($currentCategoryId, explode(',', $map['categoryids'])))) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'list') && ($map['page'] == "1+") && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        foreach ($paginationMapping as $map) {
                            if (($map['mode'] == 'gridlist') && ($map['page'] == "1+") && ($map['categoryids'] == "ALL")) {
                                return $map['productnumber'];
                            }
                        }

                        return $this->getConfig('frontend/list_per_page', 'catalog');
                    }
                } else {
                    return $this->getConfig('frontend/list_per_page', 'catalog');
                }

                break;

            default:
                break;
        }
    }

    /**
     * @param $config_path
     * @param string $section
     * @return mixed
     */
    public function getConfig($config_path, $section = 'filoblu_categorypagination_section')
    {
        return $this->scopeConfig->getValue(
            $section . '/' . $config_path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return bool
     */
    public function isFilterApplied()
    {
        $layer = $this->layerResolver->get();
        $activeFilters = $layer->getState()->getFilters();
        return ($activeFilters && count($activeFilters) > 0);
    }

    /**
     * @return mixed
     */
    public function getPageSizeForCategoryList()
    {
        $categoryId = -1;

        // to avoid problems on search
        if ($this->registry->registry('current_category')) {
            $categoryId = $this->registry->registry('current_category')->getId();
        }

        return $this->getProductNumber('list', (int)$this->request->getParam('p', 1), $categoryId);
    }

    /**
     * @return int
     */
    public function getCountPaginationMapping()
    {
        $paginationMapping = unserialize($this->getConfig('general/pagination_mapping')??'');

        if ($paginationMapping) {
            return count($paginationMapping);
        }

        return 0;
    }

}
